package PlayBot::sites::soundcloud;

use strict;
use warnings;
use utf8;

use LWP::UserAgent;
use JSON;
use Scalar::Util qw(looks_like_number);

use PlayBot::utils::Logging;

my $log = PlayBot::utils::Logging->new('STDOUT', 1);

my $root = 'http://api.soundcloud.com';
my $clientId = 'f4956716fe1a9dc9c3725af822963365';


sub regex_playlist
{
    return qr#(?:^|[^!])https?://(?:www\.)?soundcloud.com/([a-zA-Z0-9_-]+/sets/[a-zA-Z0-9_-]+)(?:\?.+)?#;
}


sub regex 
{
    return qr#(?:^|[^!])https?://(?:www\.)?soundcloud.com/([a-zA-Z0-9_-]+/[a-zA-Z0-9_-]+)(?:\?.+)?#;
}


sub get_playlist {
    shift;
    my $id = shift;

	my $ua = LWP::UserAgent->new(
        timeout => 30,
        env_proxy   => 1,
    );

    my $response;
    if (looks_like_number($id))
    {
	    $response = $ua->get($root.'/playlists/'.$id.'?client_id='.$clientId);
    }
    else
    {
	    my $url = 'https://www.soundcloud.com/'.$id;
	    $response = $ua->get($root.'/resolve.json?url='.$url.'&client_id='.$clientId);
    }
	die($response->status_line) unless ($response->is_success);

	my $content = decode_json($response->decoded_content);
    my $infos = {
	    author      => $content->{'user'}->{'username'},
        external_id => $content->{'id'},
        duration    => $content->{'track_count'},
        site        => 'soundcloud',
        title       => $content->{'title'},
	    url         => $content->{permalink_url},
    };

    my @urls;
    foreach my $item (@{ $content->{'tracks'} })
    {
        push @urls, $item->{'permalink_url'};
    }

    $infos->{'urls'} = \@urls;
	return %$infos;
}


sub get {
    shift;
	my $id = shift;

	my $ua = LWP::UserAgent->new(
        timeout => 30,
        env_proxy   => 1,
    );

    my $response;
    $log->info("the id is ".$id);

    my $retries = 0;
    while ($retries < 10) {
	$log->info("Soundcloud API is half break, try number ".($retries + 1));

        if (looks_like_number($id))
        {
	    $response = $ua->get($root.'/tracks/'.$id.'?client_id='.$clientId);
        }
        else
        {
	    my $url = 'https://www.soundcloud.com/'.$id;
	    $log->info("the url is ".$root.'/resolve.json?url='.$url.'&client_id='.$clientId);
	    $response = $ua->get($root.'/resolve.json?url='.$url.'&client_id='.$clientId);
        }
	
	if ($response->code == 404)
	{
	    $retries++;
	    sleep(1);
	}
	else
	{
	    last;
	}
    }
    die($response->status_line) unless ($response->is_success);

	my $content = decode_json($response->decoded_content);
    my $infos = {
	    author      => $content->{'user'}->{'username'},
        external_id => $content->{'id'},
        duration    => int(0.5 + $content->{'duration'} / 1000),
        site        => 'soundcloud',
        title       => $content->{'title'},
	    url         => $content->{permalink_url},
    };

	if ($content->{'downloadable'}) {
		$infos->{'ddl'} = $content->{'download_url'};
	}
    
	return %$infos;
}

1;
